Class: Merb::Cookies
Instance Method Summary collapse
-
#[]=(key, value) ⇒ Object
Implicit assignment of cookie key and value.
-
#delete(name, options = {}) ⇒ Object
Removes the cookie on the client machine by setting the value to an empty string and setting its expiration date into the past.
-
#extract_headers(controller_defaults = {}) ⇒ Object
Generate any necessary headers.
-
#initialize(constructor = {}) ⇒ Cookies
constructor
A new instance of Cookies.
-
#set_cookie(name, value, options = {}) ⇒ Object
Explicit assignment of cookie key, value and options.
Constructor Details
#initialize(constructor = {}) ⇒ Cookies
Returns a new instance of Cookies.
5 6 7 8 9 |
# File 'lib/merb-core/dispatch/cookies.rb', line 5 def initialize(constructor = {}) = Mash.new = { "domain" => Merb::Controller., "path" => '/' } super constructor end |
Instance Method Details
#[]=(key, value) ⇒ Object
Implicit assignment of cookie key and value.
Parameters
- name<~to_s>
-
Name of the cookie.
- value<~to_s>
-
Value of the cookie.
Notes
By using this method, a cookie key is marked for being included in the Set-Cookie response header.
20 21 22 23 |
# File 'lib/merb-core/dispatch/cookies.rb', line 20 def []=(key, value) [key] ||= {} super end |
#delete(name, options = {}) ⇒ Object
Removes the cookie on the client machine by setting the value to an empty string and setting its expiration date into the past.
Parameters
- name<~to_s>
-
Name of the cookie to delete.
- options<Hash>
-
Additional options to pass to
set_cookie.
52 53 54 |
# File 'lib/merb-core/dispatch/cookies.rb', line 52 def delete(name, = {}) (name, "", .merge("expires" => Time.at(0))) end |
#extract_headers(controller_defaults = {}) ⇒ Object
Generate any necessary headers.
Returns
- Hash
-
The headers to set, or an empty array if no cookies are set.
60 61 62 63 64 65 66 67 68 69 70 71 72 73 74 75 76 77 78 |
# File 'lib/merb-core/dispatch/cookies.rb', line 60 def extract_headers(controller_defaults = {}) defaults = .merge(controller_defaults) = [] self.each do |name, value| # Only set cookies that marked for inclusion in the response header. next unless [name] = defaults.merge([name]) if (expiry = ["expires"]).respond_to?(:gmtime) ["expires"] = expiry.gmtime.strftime(Merb::Const::COOKIE_EXPIRATION_FORMAT) end secure = .delete("secure") kookie = "#{name}=#{Merb::Request.escape(value)}; " # WebKit in particular doens't like empty cookie options - skip them. .each { |k, v| kookie << "#{k}=#{v}; " unless v.blank? } kookie << 'secure' if secure << kookie.rstrip end .empty? ? {} : { 'Set-Cookie' => } end |
#set_cookie(name, value, options = {}) ⇒ Object
Explicit assignment of cookie key, value and options
Parameters
- name<~to_s>
-
Name of the cookie.
- value<~to_s>
-
Value of the cookie.
- options<Hash>
-
Additional options for the cookie (see below).
Options (options)
- :path<String>
-
The path for which this cookie applies. Defaults to “/”.
- :expires<Time>
-
Cookie expiry date.
- :domain<String>
-
The domain for which this cookie applies.
- :secure<Boolean>
-
Security flag.
Notes
By using this method, a cookie key is marked for being included in the Set-Cookie response header.
41 42 43 44 |
# File 'lib/merb-core/dispatch/cookies.rb', line 41 def (name, value, = {}) [name] = self[name] = value end |